/*
 *  Copyright 2006 Columbia University.
 *
 *  This file is part of MEAPsoft.
 *
 *  MEAPsoft is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  MEAPsoft is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with MEAPsoft; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 *  02110-1301 USA
 *
 *  See the file "COPYING" for the text of the license.
 */

package com.meapsoft.gui;

import java.awt.Color;
import java.awt.Container;
import java.awt.Cursor;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import com.meapsoft.EDLFile;
import com.meapsoft.FeatExtractor;
import com.meapsoft.FeatFile;
import com.meapsoft.RTSI;
import com.meapsoft.Segmenter;
import com.meapsoft.Synthesizer;
import com.meapsoft.composers.BlipComposer;
import com.meapsoft.composers.Composer;
import com.meapsoft.composers.IntraChunkShuffleComposer;
import com.meapsoft.composers.MashupComposer;
import com.meapsoft.composers.MeapaeMComposer;
import com.meapsoft.composers.NNComposer;
import com.meapsoft.composers.SortComposer;
import com.meapsoft.featextractors.FeatureExtractor;


/**
 * GUI interface for the MEAPsoft utilities.  
 *
 * @author Douglas Repetto (douglas@music.columbia.edu)
 */

public class MEAPsoftGUI extends JFrame implements ActionListener
{
    // \todo{Use object constructors for the various utilities instead
    // of constructing command line args}

	JPanel panel;
	
	//IO data
	//String cwd;
	String dataDirectory;
	String dataBaseName = "temp";
		//segmenter
	String inputSoundFileNameFull;
	String inputSoundFileNameShort;
	String outputSegmentsFileName = dataBaseName + ".seg";
		//features
	String inputSegmentsFileName = dataBaseName + ".seg";
	String outputFeaturesFileName = dataBaseName + ".feat";
		//composer
	String inputFeaturesFileName = dataBaseName + ".feat";
	String outputEDLFileName = dataBaseName + ".edl";
		//synth
	String inputEDLFileName = dataBaseName + ".edl";
	String outputSoundFileNameFull;
	String outputSoundFileNameShort;

    // Objects passed around from utility to utility
    FeatFile segmentFile = null;
    FeatFile featFile = null;
    EDLFile edlFile = null;
	
	//tabbed GUI
	JTabbedPane tabs;
	JPanel[] stripes = new JPanel[5];
	
	//segmenter GUI
	JPanel segmenterPanel;
	JCheckBox enableSegmenterBox;
	JRadioButton oldStyleButton;
	JRadioButton beatStyleButton;
	JCheckBox firstFrameBox;
	JTextField segmenterInputSoundFileField;
	JLabel segmenterOutputSegFileLabel;
	JSlider segmentThresholdSlider;
	
	//feature extractor GUI
	JPanel featExtractorPanel;
	JCheckBox enableFeatExtractorBox;
	Vector featureCheckBoxes;
	Vector featureDescriptions;
	JLabel inputSegmentsFileLabel;
	JLabel outputFeaturesFileLabel;
	JTextField featInputSegmentsFileField;
	JTextField featOutputFeaturesFileField;
	JButton displayFeaturesButton;
	
	//composers
	String selectedComposer;
	JPanel composersPanel;
	JPanel selectComposerPanel;
	JPanel composerControlsPanel;
	Vector composerControlsPanels;
	JCheckBox enableComposersBox;
    // EDL commands
    JCheckBox composersReverseChunks;
    JCheckBox composersFadeInOutChunks;
    JCheckBox composersCrossfadeChunks;
    JSlider composersFadeDurationSlider;
	//Vector composerRadioButtons;
	JRadioButton enableSortComposerButton;
	JCheckBox reverseSortComposerBox;
	JRadioButton enableNNComposerButton;
	JRadioButton enableBLComposerButton;
	JRadioButton enableMUComposerButton;
	JRadioButton enableMMComposerButton;
	JRadioButton enableICSComposerButton;
	String targetFeaturesNameFull;
	String targetFeaturesNameShort;
	JLabel composersInputFileNameLabel;
	JLabel composersOutputFileNameLabel;
	JTextField mashupTargetFileField;
	JTextField intraChunkShuffleNumChunksField;
	
	JButton displayComposerFeaturesButton;	


	//synthesizer
	JPanel synthPanel;
	String lastEDLFileName;
	JCheckBox enableSynthBox;
	JLabel synthFileNameLabel;
	JTextField outputFileNameField;
	JButton listenButton;
	
	//prefs
	JPanel prefsPanel;
	JTextField dataBaseNameField;
	
	//start button
	JButton startButton;
	
	
	private static String slash = System.getProperty("file.separator");
	private static int OPEN = 0;
	private static int SAVE = 1;
	private static int TARGET = 2;
	private static int DIR = 3;
	
	public MEAPsoftGUI()
	{		
        super("MEAPsoft");
        
		try
		{
			File dir = new File(System.getProperty("user.dir"));
			String cwd = dir.getCanonicalPath();
			dataDirectory = cwd + slash + 
				"data";
			//System.out.println("dataDir: " + dataDirectory);
		}
		catch (IOException e)
		{
			System.out.println("can't find cwd???");
			e.printStackTrace();
		}	
		
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setResizable(false);
		BuildGUI();
		show();

        UpdateInfoTexts();
	}
	
	private void BuildGUI()
	{
		////////////////////
		// meta-GUI setup
		////////////////////

		panel = new JPanel();
		BoxLayout bl = new BoxLayout(panel, BoxLayout.Y_AXIS);
		panel.setLayout(bl);
		setContentPane(panel);

		Color c = new Color((int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127));
					
		JPanel stripe = new JPanel();
		stripe.setBackground(c);
		
		for (int i = 0; i < 5; i++)
		{
			JPanel p = new JPanel();
			p.add(new JLabel("      MEAP!      "));
			stripes[i] = p;
			stripe.add(p);
		}
		
		panel.add(stripe);
					
		tabs = new JTabbedPane();
		tabs.setBackground(c);
		panel.add(tabs);		
		

		////////////////////////////
		// now do GUIs for the tabs
		////////////////////////////
		
		BuildSegmenterGUI();
		BuildFeatureExtractorsGUI();
		BuildComposersGUI();
		BuildSynthesizerGUI();
		BuildPrefsGUI();
		
        pack();
        

		/////////////////////
		//go button!
		/////////////////////
		//this appears at the bottom of the screen, but we do it here to use
		//the same color as the rest of the main screen
		JPanel goPanel = new JPanel();
		goPanel.setBackground(c);
		
		startButton = new JButton("go!");
		startButton.setBackground(c);
		startButton.addActionListener(this);
		startButton.setActionCommand("go");
		goPanel.add(startButton);
				
		panel.add(goPanel);	

		pack();
	}
	
	private void BuildSegmenterGUI()
	{
		segmenterPanel = new JPanel();
		Color c = new Color((int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127));
		stripes[0].setBackground(c);
		segmenterPanel.setBackground(c);
		BoxLayout sbl = new BoxLayout(segmenterPanel, BoxLayout.Y_AXIS);
		segmenterPanel.setLayout(sbl);
		
		
		JPanel segmenterEnablePanel = new JPanel();
		segmenterEnablePanel.setBackground(c);
		
		enableSegmenterBox = new JCheckBox("ENABLE SEGMENTER");
		enableSegmenterBox.setBackground(c);
		enableSegmenterBox.setSelected(true);
		segmenterEnablePanel.add(enableSegmenterBox);
				
		segmenterPanel.add(segmenterEnablePanel);
		
		JPanel segmenterControlsPanel = new JPanel();
		segmenterControlsPanel.setBackground(c);
		TitledBorder title = BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"Segmenter Controls");
		title.setTitleJustification(TitledBorder.CENTER);
		segmenterControlsPanel.setBorder(title);
		BoxLayout scpbl = new BoxLayout(segmenterControlsPanel, BoxLayout.Y_AXIS);
		segmenterControlsPanel.setLayout(scpbl);
			
		JPanel inputFileNamePanel = new JPanel();
		inputFileNamePanel.setBackground(c);
		
		JLabel inputFileNameBoxLabel = new JLabel("input sound file:");
		inputFileNamePanel.add(inputFileNameBoxLabel);
	
		segmenterInputSoundFileField = new JTextField("select a .wav file!");
		segmenterInputSoundFileField.setColumns(20);
		inputFileNamePanel.add(segmenterInputSoundFileField);
		
		JButton inputBrowseButton = new JButton("browse");
		inputBrowseButton.setBackground(c);
		inputBrowseButton.addActionListener(this);
		inputBrowseButton.setActionCommand("setInputFile");
		inputFileNamePanel.add(inputBrowseButton);
		
		segmenterControlsPanel.add(inputFileNamePanel);
		
		JPanel thresholdPanel = new JPanel();
		thresholdPanel.setBackground(c);
		JLabel thresholdLabel = new JLabel("segment sensitivity: ");
		thresholdPanel.add(thresholdLabel);
		segmentThresholdSlider = new JSlider(JSlider.HORIZONTAL, 0, 12, 1);
		segmentThresholdSlider.setBackground(c);
		segmentThresholdSlider.setValue(8);
		Hashtable labelTable = new Hashtable();
		labelTable.put( new Integer(0), new JLabel("low") );
		labelTable.put( new Integer(12), new JLabel("high") );
		segmentThresholdSlider.setLabelTable( labelTable );
		segmentThresholdSlider.setPaintLabels(true);
		segmentThresholdSlider.setMajorTickSpacing(1);
		segmentThresholdSlider.setPaintTicks(true);
		thresholdPanel.add(segmentThresholdSlider);
		
		segmenterControlsPanel.add(thresholdPanel);
		
		JPanel detectorTypePanel = new JPanel();
		detectorTypePanel.setBackground(c);
		ButtonGroup onsetDetectorTypeGroup = new ButtonGroup();
		oldStyleButton = new JRadioButton("detect events");
		oldStyleButton.setBackground(c);
		beatStyleButton = new JRadioButton("detect beats");
		beatStyleButton.setBackground(c);
		onsetDetectorTypeGroup.add(oldStyleButton);
		onsetDetectorTypeGroup.add(beatStyleButton);
		detectorTypePanel.add(oldStyleButton);
		detectorTypePanel.add(beatStyleButton);
		beatStyleButton.setSelected(true);
		segmenterControlsPanel.add(detectorTypePanel);
		
		firstFrameBox = new JCheckBox("1st event = track start");
		firstFrameBox.setBackground(c);
		firstFrameBox.setSelected(true);
		segmenterControlsPanel.add(firstFrameBox);
	
		JPanel outputSegFileNamePanel = new JPanel();
		outputSegFileNamePanel.setBackground(c);
		
		JLabel sFNL = new JLabel("output segment file: ");
		outputSegFileNamePanel.add(sFNL);
		segmenterOutputSegFileLabel = new JLabel(" temp.seg ");
		segmenterOutputSegFileLabel.setOpaque(true);
		segmenterOutputSegFileLabel.setBackground(c.darker());
		outputSegFileNamePanel.add(segmenterOutputSegFileLabel);
		segmenterControlsPanel.add(outputSegFileNamePanel);
		
		segmenterPanel.add(segmenterControlsPanel);
		tabs.addTab("segmenter", segmenterPanel);
		tabs.setBackgroundAt(0, c);
	}
	
	private void BuildFeatureExtractorsGUI()
	{
		featureDescriptions = new Vector();
		
		featExtractorPanel = new JPanel();
		Color c = new Color((int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127));
		stripes[1].setBackground(c);
		featExtractorPanel.setBackground(c);
		BoxLayout fbl = new BoxLayout(featExtractorPanel, BoxLayout.Y_AXIS);
		featExtractorPanel.setLayout(fbl);
				
		Vector classNames = SniffFeatureExtractors();
		featureCheckBoxes = new Vector();
		
		JPanel featExtractorEnablePanel = new JPanel();
		featExtractorEnablePanel.setBackground(c);
		
		enableFeatExtractorBox = new JCheckBox("ENABLE FEATURE EXTRACTOR");
		enableFeatExtractorBox.setBackground(c);
		enableFeatExtractorBox.setSelected(true);
		featExtractorEnablePanel.add(enableFeatExtractorBox);
		
		featExtractorPanel.add(featExtractorEnablePanel);
		
		JPanel featExtractorControlsPanel = new JPanel();
		featExtractorControlsPanel.setBackground(c);
		BoxLayout fecp = new BoxLayout(featExtractorControlsPanel, BoxLayout.Y_AXIS);
		featExtractorControlsPanel.setLayout(fecp);
		
		JPanel extractorInputFNPanel = new JPanel();
		extractorInputFNPanel.setBackground(c);
		JLabel fEINL = new JLabel("input segments file: ");
		extractorInputFNPanel.add(fEINL);
		inputSegmentsFileLabel = new JLabel(" temp.seg ");
		inputSegmentsFileLabel.setOpaque(true);
		inputSegmentsFileLabel.setBackground(c.darker());
		extractorInputFNPanel.add(inputSegmentsFileLabel);
		featExtractorControlsPanel.add(extractorInputFNPanel);
		
		JPanel selectFeaturesPanel = new JPanel();
		selectFeaturesPanel.setBackground(c);
		TitledBorder title = BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"Feature Extractor Selector");
		title.setTitleJustification(TitledBorder.CENTER);
		selectFeaturesPanel.setBorder(title);
		BoxLayout sfp = new BoxLayout(selectFeaturesPanel, BoxLayout.Y_AXIS);
		selectFeaturesPanel.setLayout(sfp);
		
		Vector selectFeaturesPanels = new Vector();
		int numFeatures = classNames.size();
		
		for (int i = 0; i < numFeatures/4 + 1; i++)
		{
			JPanel panel = new JPanel();
			panel.setBackground(c);
			selectFeaturesPanels.add(panel);
			
			selectFeaturesPanel.add(panel);
		}
		
		featExtractorControlsPanel.add(selectFeaturesPanel);
		
		for (int i = 0; i < classNames.size(); i++)
		{
			String name = (String)classNames.elementAt(i);
			//not using actions, we'll just check their states 
			//when the GO! button is hit...
			JCheckBox cb = new JCheckBox(name);
			featureCheckBoxes.add(cb);
			cb.setBackground(c);
			cb.setToolTipText((String)featureDescriptions.elementAt(i));
			
			int whichPanel = i/4;
			
			JPanel panel = (JPanel)selectFeaturesPanels.elementAt(whichPanel);
			panel.add(cb);
		}
		
		JPanel extractorOutputFNPanel = new JPanel();
		extractorOutputFNPanel.setBackground(c);
		JLabel fEONL = new JLabel("output features file: ");
		extractorOutputFNPanel.add(fEONL);
		outputFeaturesFileLabel = new JLabel(" temp.feat ");
		outputFeaturesFileLabel.setOpaque(true);
		outputFeaturesFileLabel.setBackground(c.darker());
		extractorOutputFNPanel.add(outputFeaturesFileLabel);
		featExtractorControlsPanel.add(extractorOutputFNPanel);
		
		JPanel displayFeaturesPanel = new JPanel();
		displayFeaturesPanel.setBackground(c);
		displayFeaturesButton = new JButton("display extracted features");
		displayFeaturesButton.setEnabled(false);
		displayFeaturesButton.addActionListener(this);
		displayFeaturesButton.setActionCommand("displayFeatures");
		displayFeaturesButton.setBackground(c);
		displayFeaturesPanel.add(displayFeaturesButton);
		featExtractorControlsPanel.add(displayFeaturesPanel);
		
		featExtractorPanel.add(featExtractorControlsPanel);
		tabs.addTab("feature extractors", featExtractorPanel);
		tabs.setBackgroundAt(1, c);
	}
	
	private void BuildComposersGUI()
	{
		composerControlsPanels = new Vector();
		
		composersPanel = new JPanel();
		Color c = new Color((int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127));
		stripes[2].setBackground(c);
		composersPanel.setBackground(c);
		BoxLayout cbl = new BoxLayout(composersPanel, BoxLayout.Y_AXIS);
		composersPanel.setLayout(cbl);
		
		JPanel enableComposersPanel = new JPanel();
		enableComposersPanel.setBackground(c);
		
		enableComposersBox = new JCheckBox("ENABLE COMPOSERS");
		enableComposersBox.setBackground(c);
		enableComposersBox.setSelected(true);
		enableComposersPanel.add(enableComposersBox);

		composersPanel.add(enableComposersPanel);

		JPanel composersInputFileNamePanel = new JPanel();
		composersInputFileNamePanel.setBackground(c);
		JLabel cINL = new JLabel("input features file: ");
		composersInputFileNamePanel.add(cINL);
		composersInputFileNameLabel = new JLabel(" temp.feat ");
		composersInputFileNameLabel.setOpaque(true);
		composersInputFileNameLabel.setBackground(c.darker());
		composersInputFileNamePanel.add(composersInputFileNameLabel);
		composersPanel.add(composersInputFileNamePanel);

        
		selectComposerPanel = new JPanel();
		selectComposerPanel.setBackground(c);
		selectComposerPanel.add(new JLabel("select a composer: "));
		
		
		ButtonGroup composerButtons = new ButtonGroup();
		
		composerControlsPanels.add(BuildSortComposerGUI(c, composerButtons));
		composerControlsPanels.add(BuildNearestNeighborComposerGUI(c, composerButtons));
		composerControlsPanels.add(BuildBlipComposerGUI(c, composerButtons));
		composerControlsPanels.add(BuildMashupComposerGUI(c, composerButtons));	
		composerControlsPanels.add(BuildMeapaeMComposerGUI(c, composerButtons));	
		composerControlsPanels.add(BuildIntraChunkShuffleComposerGUI(c, composerButtons));
		
		composersPanel.add(selectComposerPanel);

		composerControlsPanel = new JPanel();
		composerControlsPanel.setBackground(c);

		composerControlsPanel.add((JPanel)composerControlsPanels.elementAt(0));

		selectedComposer = "SortComposer";
		
		composersPanel.add(composerControlsPanel);		

        JPanel chunkPanel = new JPanel(new GridLayout(2, 1));
		chunkPanel.setBackground(c);
       	TitledBorder title = BorderFactory.createTitledBorder(
            BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
            "Universal Chunk Operations");
        title.setTitleJustification(TitledBorder.CENTER);
        chunkPanel.setBorder(title);
        JPanel chunkCommandPanel = new JPanel();
		chunkCommandPanel.setBackground(c);
        composersReverseChunks = new JCheckBox("reverse");
        composersReverseChunks.setBackground(c);
		composersReverseChunks.setToolTipText("Reverse audio in each chunk");
        chunkCommandPanel.add(composersReverseChunks);
		composersFadeInOutChunks = new JCheckBox("apply fade in/out");
		composersFadeInOutChunks.addActionListener(this);
        composersFadeInOutChunks.setBackground(c);
		composersFadeInOutChunks.setToolTipText("Fade in/out on each chunk of audio");
        composersFadeInOutChunks.setActionCommand("fade");
        chunkCommandPanel.add(composersFadeInOutChunks);
        composersCrossfadeChunks = new JCheckBox("crossfade");
		composersCrossfadeChunks.addActionListener(this);
        composersCrossfadeChunks.setBackground(c);
		composersCrossfadeChunks.setToolTipText("Overlap fades from chunk to chunk");
        composersCrossfadeChunks.setActionCommand("fade");
        chunkCommandPanel.add(composersCrossfadeChunks);
        chunkPanel.add(chunkCommandPanel);
        JPanel crossfadePanel = new JPanel();
        crossfadePanel.setBackground(c);
		JLabel crossfadeLabel = new JLabel("fade length (ms): ");
        crossfadePanel.add(crossfadeLabel);
        composersFadeDurationSlider = new JSlider(JSlider.HORIZONTAL, 0, 50, 0);
		composersFadeDurationSlider.setBackground(c);
		composersFadeDurationSlider.setToolTipText("Duration of fades");
		composersFadeDurationSlider.setValue(0);
		composersFadeDurationSlider.setEnabled(false);
		composersFadeDurationSlider.setPaintLabels(true);
		composersFadeDurationSlider.setMajorTickSpacing(10);
		composersFadeDurationSlider.setMinorTickSpacing(2);
		composersFadeDurationSlider.setPaintTicks(true);
        crossfadePanel.add(composersFadeDurationSlider);
        chunkPanel.add(crossfadePanel);

		composersPanel.add(chunkPanel);

		JPanel composersOutputFileNamePanel = new JPanel();
		composersOutputFileNamePanel.setBackground(c);
		JLabel cONL = new JLabel("output edl file: ");
		composersOutputFileNamePanel.add(cONL);
		composersOutputFileNameLabel = new JLabel(" temp.edl ");
		composersOutputFileNameLabel.setOpaque(true);
		composersOutputFileNameLabel.setBackground(c.darker());
		composersOutputFileNamePanel.add(composersOutputFileNameLabel);
		
		composersPanel.add(composersOutputFileNamePanel);

		JPanel displayComposerFeaturesPanel = new JPanel();
		displayComposerFeaturesPanel.setBackground(c);
		displayComposerFeaturesButton = new JButton("display composed features");
		displayComposerFeaturesButton.setEnabled(false);
		displayComposerFeaturesButton.addActionListener(this);
		displayComposerFeaturesButton.setActionCommand("displayComposerFeatures");
		displayComposerFeaturesButton.setBackground(c);
		displayComposerFeaturesPanel.add(displayComposerFeaturesButton);
		
		composersPanel.add(displayComposerFeaturesPanel);
		
		panel.add(composersPanel);

		tabs.addTab("composers", composersPanel);
		tabs.setBackgroundAt(2, c);
	}
	
	private JPanel BuildSortComposerGUI(Color c, ButtonGroup composerButtons)
	{		
		//sort composer
		JPanel sortComposerPanel = new JPanel();
		BoxLayout bl = new BoxLayout(sortComposerPanel, BoxLayout.Y_AXIS);
		sortComposerPanel.setLayout(bl);
		sortComposerPanel.setBackground(c);

		enableSortComposerButton = new JRadioButton("simple sort");
		enableSortComposerButton.setBackground(c);
		composerButtons.add(enableSortComposerButton);
		enableSortComposerButton.setSelected(true);
		enableSortComposerButton.addActionListener(this);
		//set the action command to the name of the composer class!
		enableSortComposerButton.setActionCommand("SortComposer");
		selectComposerPanel.add(enableSortComposerButton);
		
		JTextArea description = new JTextArea(SortComposer.description);
		description.setColumns(50);
		description.setLineWrap(true);
		description.setWrapStyleWord(true);
		description.setBackground(c);
		description.setEditable(false);
		sortComposerPanel.add(description);
		
		JPanel controlsPanel = new JPanel();
		controlsPanel.setBackground(c);
		TitledBorder title = BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"SimpleSort Controls");
		title.setTitleJustification(TitledBorder.CENTER);
		controlsPanel.setBorder(title);
		reverseSortComposerBox = new JCheckBox("reverse sort");
		reverseSortComposerBox.setBackground(c);
		controlsPanel.add(reverseSortComposerBox);
		
		sortComposerPanel.add(controlsPanel);
		
		

		
		//selectComposerPanel.add(sortComposerPanel);
		return sortComposerPanel;
	}
	
	private JPanel BuildNearestNeighborComposerGUI(Color c, ButtonGroup composerButtons)
	{
		//nearest neighbor composer
		JPanel nNComposerPanel = new JPanel();
		nNComposerPanel.setBackground(c);
		enableNNComposerButton = new JRadioButton("nearest neighbor");
		enableNNComposerButton.setBackground(c);
		composerButtons.add(enableNNComposerButton);
		enableNNComposerButton.setSelected(false);
		enableNNComposerButton.addActionListener(this);
		//set the action command to the name of the composer class!
		enableNNComposerButton.setActionCommand("NNComposer");
		selectComposerPanel.add(enableNNComposerButton);
		
		JTextArea description = new JTextArea(NNComposer.description);
		description.setColumns(50);
		description.setLineWrap(true);
		description.setWrapStyleWord(true);
		description.setBackground(c);
		description.setEditable(false);
		nNComposerPanel.add(description);
		
		
		return nNComposerPanel;
	}
	
	private JPanel BuildBlipComposerGUI(Color c, ButtonGroup composerButtons)
	{
		
		// blip composer
		JPanel bLComposerPanel = new JPanel();
		bLComposerPanel.setBackground(c);
		enableBLComposerButton = new JRadioButton("add blips");
		enableBLComposerButton.setBackground(c);
		composerButtons.add(enableBLComposerButton);
		enableBLComposerButton.setSelected(false);
		enableBLComposerButton.addActionListener(this);
		//set the action command to the name of the composer class!
		enableBLComposerButton.setActionCommand("BlipComposer");
		selectComposerPanel.add(enableBLComposerButton);
		
		JTextArea description = new JTextArea(BlipComposer.description);
		description.setColumns(50);
		description.setLineWrap(true);
		description.setWrapStyleWord(true);
		description.setBackground(c);
		description.setEditable(false);
		bLComposerPanel.add(description);
		
		return bLComposerPanel;
	}
	
	private JPanel BuildMashupComposerGUI(Color c, ButtonGroup composerButtons)
	{
		//mashup composer
		JPanel mUComposerPanel = new JPanel();
		BoxLayout bl = new BoxLayout(mUComposerPanel, BoxLayout.Y_AXIS);
		mUComposerPanel.setLayout(bl);
		mUComposerPanel.setBackground(c);
		enableMUComposerButton = new JRadioButton("mashup!");
		enableMUComposerButton.setBackground(c);
		composerButtons.add(enableMUComposerButton);
		enableMUComposerButton.setSelected(false);
		enableMUComposerButton.addActionListener(this);
		//set the action command to the name of the composer class!
		enableMUComposerButton.setActionCommand("MashupComposer");
		selectComposerPanel.add(enableMUComposerButton);
		
		JTextArea description = new JTextArea(MashupComposer.description);
		description.setColumns(50);
		description.setLineWrap(true);
		description.setWrapStyleWord(true);
		description.setBackground(c);
		description.setEditable(false);
		mUComposerPanel.add(description);
		
		JPanel controlsPanel = new JPanel();
		controlsPanel.setBackground(c);
		TitledBorder title = BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"Mashup Controls");
		title.setTitleJustification(TitledBorder.CENTER);
		controlsPanel.setBorder(title);
		
		JLabel mULabel = new JLabel("target features file:");
		controlsPanel.add(mULabel);
		mashupTargetFileField = new JTextField("target .feat file");
		mashupTargetFileField.setColumns(20);
		mashupTargetFileField.setEditable(false);
		controlsPanel.add(mashupTargetFileField);
		JButton mUBrowseButton = new JButton("browse");
		mUBrowseButton.setBackground(c);
		mUBrowseButton.addActionListener(this);
		mUBrowseButton.setActionCommand("setMUTargetFeaturesFile");
		controlsPanel.add(mUBrowseButton);
		
		mUComposerPanel.add(controlsPanel);

		//selectComposerPanel.add(mUComposerPanel);
		return mUComposerPanel;
	}

	private JPanel BuildMeapaeMComposerGUI(Color c, ButtonGroup composerButtons)
	{
		
		// MeapaeM composer
		JPanel MeapaeMComposerPanel = new JPanel();
		MeapaeMComposerPanel.setBackground(c);
		enableMMComposerButton = new JRadioButton("MeapaeM");
		enableMMComposerButton.setBackground(c);
		composerButtons.add(enableMMComposerButton);
		enableMMComposerButton.setSelected(false);
		enableMMComposerButton.addActionListener(this);
		//set the action command to the name of the composer class!
		enableMMComposerButton.setActionCommand("MeapaeMComposer");
		selectComposerPanel.add(enableMMComposerButton);
		
		JTextArea description = new JTextArea(MeapaeMComposer.description);
		description.setColumns(50);
		description.setLineWrap(true);
		description.setWrapStyleWord(true);
		description.setBackground(c);
		description.setEditable(false);
		MeapaeMComposerPanel.add(description);
		
		return MeapaeMComposerPanel;
	}
	
	private JPanel BuildIntraChunkShuffleComposerGUI(Color c, ButtonGroup composerButtons)
	{
		
		// IntraChunkShuffle composer
		JPanel IntraChunkShuffleComposerPanel = new JPanel();
		IntraChunkShuffleComposerPanel.setBackground(c);
		BoxLayout bl = new BoxLayout(IntraChunkShuffleComposerPanel, BoxLayout.Y_AXIS);
		IntraChunkShuffleComposerPanel.setLayout(bl);
		enableICSComposerButton = new JRadioButton("IntraChunkShuffle");
		enableICSComposerButton.setBackground(c);
		composerButtons.add(enableICSComposerButton);
		enableICSComposerButton.setSelected(false);
		enableICSComposerButton.addActionListener(this);
		//set the action command to the name of the composer class!
		enableICSComposerButton.setActionCommand("IntraChunkShuffleComposer");
		selectComposerPanel.add(enableICSComposerButton);
		
		JTextArea description = new JTextArea(IntraChunkShuffleComposer.description);
		description.setColumns(50);
		description.setLineWrap(true);
		description.setWrapStyleWord(true);
		description.setBackground(c);
		description.setEditable(false);
		IntraChunkShuffleComposerPanel.add(description);
		
		JPanel controlsPanel = new JPanel();
		controlsPanel.setBackground(c);
		TitledBorder title = BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"IntraChunkShuffle Controls");
		title.setTitleJustification(TitledBorder.CENTER);
		controlsPanel.setBorder(title);
		
		JLabel iCSLabel = new JLabel("number of sub chunks:");
		controlsPanel.add(iCSLabel);
		intraChunkShuffleNumChunksField = new JTextField("4");
		controlsPanel.add(intraChunkShuffleNumChunksField);
		
		IntraChunkShuffleComposerPanel.add(controlsPanel);
		
		return IntraChunkShuffleComposerPanel;
	}
	
	private void BuildSynthesizerGUI()
	{
		synthPanel = new JPanel();
		Color c = new Color((int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127));
		stripes[3].setBackground(c);
		synthPanel.setBackground(c);
		BoxLayout synthbl = new BoxLayout(synthPanel, BoxLayout.Y_AXIS);
		synthPanel.setLayout(synthbl);
		
		JPanel enableSynthPanel = new JPanel();
		enableSynthPanel.setBackground(c);
		
		enableSynthBox = new JCheckBox("ENABLE SYNTHESIZER");
		enableSynthBox.setBackground(c);
		enableSynthBox.setSelected(true);
		enableSynthPanel.add(enableSynthBox);

		synthPanel.add(enableSynthPanel);
		
		JPanel synthFileIOPanel = new JPanel();
		synthFileIOPanel.setBackground(c);
		
		JLabel sNL = new JLabel("input edl file: ");
		synthFileIOPanel.add(sNL);
		synthFileNameLabel = new JLabel(" temp.edl ");
		synthFileNameLabel.setOpaque(true);
		synthFileNameLabel.setBackground(c.darker());
		synthFileIOPanel.add(synthFileNameLabel);
		synthPanel.add(synthFileIOPanel);
		
		JPanel synthControlsPanel = new JPanel();
		synthControlsPanel.setBackground(c);
		TitledBorder title = BorderFactory.createTitledBorder(
			BorderFactory.createEtchedBorder(EtchedBorder.LOWERED), 
			"Synthesizer Controls");
		title.setTitleJustification(TitledBorder.CENTER);
		synthControlsPanel.setBorder(title);
		BoxLayout scp = new BoxLayout(synthControlsPanel, BoxLayout.Y_AXIS);
		synthControlsPanel.setLayout(scp);
		
		JPanel outputFileNamePanel = new JPanel();
		outputFileNamePanel.setBackground(c);
		
		JLabel outputFileNameBoxLabel = new JLabel("output sound file:");
		outputFileNamePanel.add(outputFileNameBoxLabel);
	
		outputFileNameField = new JTextField("output wav file");
		outputFileNameField.setColumns(20);
		outputFileNameField.setEditable(false);
		outputFileNamePanel.add(outputFileNameField);
		
		JButton outputBrowseButton = new JButton("browse");
		outputBrowseButton.setBackground(c);
		outputBrowseButton.addActionListener(this);
		outputBrowseButton.setActionCommand("setOutputFile");
		outputFileNamePanel.add(outputBrowseButton);
		
		
		listenButton = new JButton("listen");
		listenButton.setBackground(c);
		listenButton.addActionListener(this);
		listenButton.setActionCommand("listen");
		outputFileNamePanel.add(listenButton);
		
		
		synthControlsPanel.add(outputFileNamePanel);
		
		synthPanel.add(synthControlsPanel);
		tabs.addTab("synthesizer", synthPanel);
		tabs.setBackgroundAt(3, c);
	}
	
	private void BuildPrefsGUI()
	{
		prefsPanel = new JPanel();
		Color c = new Color((int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127),
					(int)(Math.random() * 127 + 127));
		stripes[4].setBackground(c);
		prefsPanel.setBackground(c);
		BoxLayout prefsBL = new BoxLayout(prefsPanel, BoxLayout.Y_AXIS);
		prefsPanel.setLayout(prefsBL);
		
		prefsPanel = new JPanel();
		prefsPanel.setBackground(c);
		JLabel dataBaseNameLabel = new JLabel("data base name: ");
		prefsPanel.add(dataBaseNameLabel);		
		dataBaseNameField = new JTextField("temp");
		dataBaseNameField.setColumns(10);
		prefsPanel.add(dataBaseNameField);
		JButton dataBaseNameUpdateButton = new JButton("update");
		dataBaseNameUpdateButton.setBackground(c);
		dataBaseNameUpdateButton.addActionListener(this);
		dataBaseNameUpdateButton.setActionCommand("dataBaseName");
		prefsPanel.add(dataBaseNameUpdateButton);
		
		tabs.addTab("preferences", prefsPanel);	

	}
	
	private Vector SniffFeatureExtractors()
	{
        Vector v = null;
        try 
        {
            v = RTSI.findnames("com.meapsoft.featextractors", Class.forName("com.meapsoft.featextractors.FeatureExtractor"));
            // \todo{where else should we look for feature extractors?}
            // also check current directory:
            Vector v2 = RTSI.findnames(System.getProperty("user.dir"), Class.forName("com.meapsoft.featextractors.FeatureExtractor")); 
            if(v2 != null)
                v.addAll(v2);
            
            for (int i = 0; i < v.size(); i++)
            {
				FeatureExtractor f = null;
				
				try
				{
					String name = "com.meapsoft.featextractors." + (String)v.elementAt(i);
					//String description = Class.forName(name).newInstance().description;
					f = (FeatureExtractor)(Class.forName(name).newInstance());
					featureDescriptions.add(f.description());
				}
				catch (InstantiationException e1)
				{
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				catch (IllegalAccessException e1)
				{
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
            }
        }
        catch(ClassNotFoundException e)
        {
            e.printStackTrace();
            return(null);
        }

        return v;
	}
	
	private String[] FileSelector(int mode)
	{		
		JFileChooser chooser = new JFileChooser();
		
		chooser.setCurrentDirectory(new File(dataDirectory));
		int returnVal = 0;
		
		if (mode == OPEN)
			returnVal = chooser.showOpenDialog(null);
		else if (mode == SAVE)
			returnVal = chooser.showSaveDialog(null);
		else if (mode == DIR)
		{
			chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			returnVal = chooser.showOpenDialog(this);
		}
		else
			return null;
			
		String[] name = new String[2];
		
		if(returnVal == JFileChooser.APPROVE_OPTION) 
		{
			try
			{
				name[0] = chooser.getCurrentDirectory().getCanonicalPath() +
					slash +
					chooser.getSelectedFile().getName();
				name[1] = chooser.getSelectedFile().getName();

			}
			catch (IOException e)
			{
				e.printStackTrace();
				return null;
			}
		}
		
		return name;
	}

    private void initSegmentFile()
    {        
        String fn = inputSoundFileNameFull;

        if(segmentFile == null || segmentFile.filename != fn)
            segmentFile = new FeatFile(fn);
    }

    private void initFeatFile()
    {
        String fn = dataDirectory + slash + outputFeaturesFileName;

        if(featFile == null || featFile.filename != fn)
            featFile = new FeatFile(fn);
    }

    private void initEDLFile()
    {        
        String fn = dataDirectory + slash + outputEDLFileName;

        if(edlFile == null || edlFile.filename != fn)
            edlFile = new EDLFile(fn);
    }

	private void DoSegmenter()
	{		
		//value will be 0-9, we want 1-5
		double bandThresh = 4.0 - (segmentThresholdSlider.getValue()/4.0);
		double bandFrac = 0.3;
		
		boolean beatOnsetDetector = beatStyleButton.isSelected();
		boolean firstFrameOnset = firstFrameBox.isSelected();
		
		String segmentsFileName = dataDirectory + slash + outputSegmentsFileName;
		
		Segmenter segmenter = new Segmenter(inputSoundFileNameFull,
			segmentsFileName, 0, bandThresh, bandFrac, beatOnsetDetector, firstFrameOnset);

        try
        {
            segmentFile = segmenter.processAudioFile(inputSoundFileNameFull);
            segmentFile.writeFile();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
	}

	private void DoFeatureExtractor()
	{
        Vector featExts = new Vector();
		for(int i = 0; i < featureCheckBoxes.size(); i++)	
        {
			JCheckBox cb = (JCheckBox)featureCheckBoxes.elementAt(i);
			
            try
            {
                if (cb.isSelected())
                    featExts.add(Class.forName("com.meapsoft.featextractors." + cb.getText())
                                 .newInstance());
            }
            catch(Exception e)
            {
                System.out.println("featextractor error");
                e.printStackTrace();
            }
        }

        FeatExtractor featExtractor = 
            new FeatExtractor(segmentFile, featFile, featExts);
	
        try
        {				
            // clear whatever chunks may have already been calculated
            featFile.clearChunks();
            
            featExtractor.setup();
            featExtractor.processFeatFiles();
            featFile.writeFile();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }

        displayFeaturesButton.setEnabled(true);
	}
	
	private int DoComposer()
	{	
		/*
		String composerName = null;
		
		for (int i = 0; i < composerRadioButtons.size(); i++)
		{
			JRadioButton rb = (JRadioButton)composerRadioButtons.elementAt(i);
			if (rb.isSelected())
				composerName = rb.getActionCommand();
		}
		
		//System.out.println("composer: " + composerName);
		*/
		
        Composer composer = null;
		if (selectedComposer.equals("SortComposer"))
		{
			composer = new SortComposer(featFile, edlFile);
            ((SortComposer)composer).setReverseSort(reverseSortComposerBox.isSelected());
		}
		else if (selectedComposer.equals("NNComposer"))
		{
			composer = new NNComposer(featFile, edlFile);
		}
		else if (selectedComposer.equals("BlipComposer"))
		{
			composer = new BlipComposer(featFile, edlFile);
		}
		else if (selectedComposer.equals("MashupComposer"))
		{
			edlFile = new EDLFile(dataDirectory + //dataDirectoryNameField.getText() + 
				slash + dataBaseName + "_using_" + 
				targetFeaturesNameShort + ".edl");

            FeatFile targetFile = new FeatFile(targetFeaturesNameFull);
				
            composer = new MashupComposer(featFile, targetFile, edlFile);
		}
		else if (selectedComposer.equals("MeapaeMComposer"))
		{
			composer = new MeapaeMComposer(featFile, edlFile);
		}
		else if (selectedComposer.equals("IntraChunkShuffleComposer"))
		{
			int numSubChunks = 4;
			
			try
			{
				numSubChunks = new Integer(intraChunkShuffleNumChunksField.getText()).intValue();
			}
			catch (Exception e)
			{
				System.out.println(e);
				return -1;
			}
			composer = new IntraChunkShuffleComposer(featFile, edlFile, numSubChunks);
		}
		else
		{
			System.out.println("I don't recognize that composer!");
			return -1;
		}
		
        lastEDLFileName = edlFile.filename; 

        double crossfade = (double)composersFadeDurationSlider.getValue()/1000;
        if(composersFadeInOutChunks.isSelected() & crossfade > 0)
            composer.addCommand("fade("+crossfade+")");
        if(composersCrossfadeChunks.isSelected() & crossfade > 0)
            composer.addCommand("crossfade("+crossfade+")");
        if(composersReverseChunks.isSelected())
            composer.addCommand("reverse");
        
        composer.go();

        displayComposerFeaturesButton.setEnabled(true);
        
        return 0;
    }
	
	private void DoSynth()
	{	
		Synthesizer synth = new Synthesizer(edlFile, outputSoundFileNameFull);
        synth.go();
	}
	
	private void UpdateFileNames()
	{
		//segmenter
		outputSegmentsFileName = dataBaseName + ".seg";
		//features
		inputSegmentsFileName = dataBaseName + ".seg";
		outputFeaturesFileName = dataBaseName + ".feat";
		//composer
		inputFeaturesFileName = dataBaseName + ".feat";
		outputEDLFileName = dataBaseName + ".edl";
		//synth
		inputEDLFileName = dataBaseName + ".edl";
	}
	
	private void UpdateInfoTexts()
	{	
		//segmenter
		//segmenterInputSoundFileField.setText(inputSoundFileNameShort);
		segmenterOutputSegFileLabel.setText(" " + outputSegmentsFileName + " ");
		//features
		inputSegmentsFileLabel.setText(" " + inputSegmentsFileName + " ");
		outputFeaturesFileLabel.setText(" " + outputFeaturesFileName + " ");
		//composer
		composersInputFileNameLabel.setText(" " + inputFeaturesFileName + " ");
		composersOutputFileNameLabel.setText(" " + outputEDLFileName + " ");
		//synth
		synthFileNameLabel.setText(" " + inputEDLFileName + " ");
		pack();	

		initSegmentFile();
		initFeatFile();
		initEDLFile();
	}
	
	public void actionPerformed(ActionEvent arg0)
	{

		String command = arg0.getActionCommand();
		//System.out.println("command: " + command);

		if (command.equals("setInputFile"))
		{
			String names[] = FileSelector(OPEN);
			
			if (names[0] == null)
				return;
			inputSoundFileNameFull = names[0];
			inputSoundFileNameShort = names[1];
			dataBaseName = inputSoundFileNameShort;
			segmenterInputSoundFileField.setText(inputSoundFileNameShort);
			UpdateFileNames();
			UpdateInfoTexts();
		}
		else if (command.equals("setOutputFile"))
		{
			String names[] = FileSelector(SAVE);
			
			if (names[0] == null)
				return;
				
			outputSoundFileNameFull = names[0];
			outputSoundFileNameShort = names[1];
			outputFileNameField.setText(outputSoundFileNameShort);
		}
		else if (command.equals("SortComposer"))
		{
			//System.out.println("adding sort composer gui...");
			selectedComposer = command;
			composerControlsPanel.removeAll();
			composerControlsPanel.add((JPanel)composerControlsPanels.elementAt(0));
			pack();
			repaint();
		}
		else if (command.equals("NNComposer"))
		{
			//System.out.println("adding NN composer gui...");
			selectedComposer = command;
			composerControlsPanel.removeAll();
			composerControlsPanel.add((JPanel)composerControlsPanels.elementAt(1));
			pack();
			repaint();
		}
		else if (command.equals("BlipComposer"))
		{
			//System.out.println("adding blip composer gui...");
			selectedComposer = command;
			composerControlsPanel.removeAll();
			composerControlsPanel.add((JPanel)composerControlsPanels.elementAt(2));
			pack();
			repaint();
		}
		else if (command.equals("MashupComposer"))
		{
			//System.out.println("adding mashup composer gui...");
			selectedComposer = command;
			composerControlsPanel.removeAll();
			composerControlsPanel.add((JPanel)composerControlsPanels.elementAt(3));
			pack();
			repaint();
		}
		else if (command.equals("MeapaeMComposer"))
		{
			//System.out.println("adding MeapaeM composer gui...");
			selectedComposer = command;
			composerControlsPanel.removeAll();
			composerControlsPanel.add((JPanel)composerControlsPanels.elementAt(4));
			pack();
			repaint();
		}
		else if (command.equals("IntraChunkShuffleComposer"))
		{
			//System.out.println("adding IntraChunkShuffleComposer composer gui...");
			selectedComposer = command;
			composerControlsPanel.removeAll();
			composerControlsPanel.add((JPanel)composerControlsPanels.elementAt(5));
			pack();
			repaint();
		}
		else if (command.equals("setMUTargetFeaturesFile"))
		{
			String names[] = FileSelector(OPEN);
			targetFeaturesNameFull = names[0];
			targetFeaturesNameShort = names[1];
			if (targetFeaturesNameShort.endsWith("feat"))
				mashupTargetFileField.setText(targetFeaturesNameShort);
			else
				mashupTargetFileField.setText("error! use only .feat files!");
		}
		else if (command.equals("dataBaseName"))
		{
			dataBaseName = dataBaseNameField.getText();
			UpdateFileNames();
			UpdateInfoTexts();
		}
		else if (command.equals("fade"))
        {
            boolean enable = composersFadeInOutChunks.isSelected() 
                || composersCrossfadeChunks.isSelected();
            composersFadeDurationSlider.setEnabled(enable);
        }
		else if (command.equals("displayFeatures") ||
			command.equals("displayComposerFeatures"))
		{
			DataDisplayPanel fp;
			
            String frameName;
			if (command.equals("displayFeatures"))
            {
				fp = new DataDisplayPanel(featFile);
                frameName = outputFeaturesFileName;
            }
			else 
            {
				fp = new DataDisplayPanel((FeatFile)edlFile);
                frameName = edlFile.filename;
            }			

 			try
 			{
 				fp.setup();

                 JFrame jframe = new JFrame(frameName);
                 jframe.setContentPane(fp);
                 jframe.pack();
                 jframe.setVisible(true);
 			}
 			catch (Exception e)
 			{
 				System.out.println("problem with ImagePanel.");
 				e.printStackTrace();
 			}
		}
		else if (command.equals("go"))
		{	
			Container c = getContentPane();  // get the window's content pane
			c.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
			
			if (enableSegmenterBox.isSelected())
			{
				if (inputSoundFileNameFull == null)
				{
					JOptionPane.showMessageDialog(this,
						"You need to pick an input file!!!");
					c.setCursor(Cursor.getDefaultCursor());
					return;
				}
				DoSegmenter();	
			}
			if (enableFeatExtractorBox.isSelected())
				DoFeatureExtractor();
			if (enableComposersBox.isSelected())
				DoComposer();
			if (enableSynthBox.isSelected())
			{
				if (outputSoundFileNameFull == null)
				{
					JOptionPane.showMessageDialog(this,
						"You need to pick an output file!!!");
					c.setCursor(Cursor.getDefaultCursor());
					return;
				}
				DoSynth();
			}
			
			c.setCursor(Cursor.getDefaultCursor());
			
			JOptionPane.showMessageDialog(this,
				"Finished MEAPing!!!");
		}
		//Bagging this, it only works on OSX. Bummer!
		
		else if (command.equals("listen"))
		{
			if (outputSoundFileNameFull == null)
			{
				JOptionPane.showMessageDialog(this,
					"You need to pick an output file!!!");
				return;
			}
			try
			{
				Process p = null;
				if (System.getProperty("os.name").equals("Mac OS X"))
					p = Runtime.getRuntime().exec("open " + outputSoundFileNameFull);
				else if (System.getProperty("os.name").equals("Linux"))
					p = Runtime.getRuntime().exec("play " + outputSoundFileNameFull);
				else
					JOptionPane.showMessageDialog(this,
						"Listen only works on OSX and Linux. Sorry!");
			}
			catch (IOException e)
			{
				e.printStackTrace();
			}
		}
	}
	

	public static void main(String[] args)
	{	
		//Schedule a job for the event-dispatching thread:
		//creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() 
			{
				new MEAPsoftGUI();
			}
		});

	}
}
